# SPDX-FileCopyrightText: 2020 Foamyguy, written for Adafruit Industries
#
# SPDX-License-Identifier: Unlicense

"""
CircuitPython example for Monster M4sk.

Draws a basic eye dot on each screen. Looks at nose
when booped. Prints acceleration and light sensor
data when booped as well.
"""

import time

import board
import displayio
from adafruit_display_shapes.circle import Circle

import adafruit_monsterm4sk

# Account for slight screen difference if you want
LEFT_Y_OFFSET = 0  # 12 # my left screen is a tad higher

SCREEN_SIZE = 240

i2c_bus = board.I2C()  # uses board.SCL and board.SDA
# i2c_bus = board.STEMMA_I2C()  # For using the built-in STEMMA QT connector on a microcontroller

mask = adafruit_monsterm4sk.MonsterM4sk(i2c=i2c_bus)

left_group = displayio.Group()
mask.left_display.root_group = left_group

right_group = displayio.Group()
mask.right_display.root_group = right_group

right_circle = Circle(SCREEN_SIZE // 2, SCREEN_SIZE // 2, 40, fill=0x0000FF)
right_group.append(right_circle)

left_circle = Circle(SCREEN_SIZE // 2, SCREEN_SIZE // 2, 40, fill=0x00AA66)
left_group.append(left_circle)

while True:
    # print(mask.boop)
    if mask.boop:
        left_circle.x = 0
        right_circle.x = SCREEN_SIZE - 40 - 40 - 2

        right_circle.y = SCREEN_SIZE // 4 - 40
        left_circle.y = SCREEN_SIZE // 4 - 40 + LEFT_Y_OFFSET
        print(mask.acceleration)
        print(mask.light)
        time.sleep(0.5)
    else:
        left_circle.x = SCREEN_SIZE // 2 - 40
        right_circle.x = SCREEN_SIZE // 2 - 40

        right_circle.y = SCREEN_SIZE // 2 - 40
        left_circle.y = SCREEN_SIZE // 2 - 40 + LEFT_Y_OFFSET



# SPDX-FileCopyrightText: Copyright (c) 2020 Foamyguy for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""
`adafruit_monsterm4sk`
================================================================================

Helper library for the Monster M4sk device. Allows usage of screens and other built-in hardware.


* Author(s): Foamyguy

Implementation Notes
--------------------

**Hardware:**

* `MONSTER M4SK <https://www.adafruit.com/product/4343>`_

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://github.com/adafruit/circuitpython/releases


# * Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
# * Adafruit's Register library: https://github.com/adafruit/Adafruit_CircuitPython_Register
"""

# imports
import time

import adafruit_lis3dh
import board
import busio
import digitalio
import displayio
import fourwire
import pwmio
import touchio
from adafruit_seesaw.seesaw import Seesaw
from adafruit_st7789 import ST7789

try:
    from typing import Dict, Optional, Union

    from busio import I2C
except ImportError:
    pass

__version__ = "1.0.8"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_MONSTERM4SK.git"

# Seesaw pin numbers
SS_LIGHTSENSOR_PIN = 2  # "through-hole" light sensor near left eye
SS_VCCSENSOR_PIN = 3
SS_BACKLIGHT_PIN = 5  # left screen backlight
SS_TFTRESET_PIN = 8  # left screen reset

# buttons above left eye. Match silkscreen :)
SS_SWITCH1_PIN = 9
SS_SWITCH2_PIN = 10
SS_SWITCH3_PIN = 11


class MonsterM4sk:
    """Represents a single Monster M4sk

    The terms "left" and "right" are always used from the
    perspective of looking out of the mask.
    The right screen is the one USB port directly above it.
    """

    def __init__(self, i2c: Optional[I2C] = None):
        """
        :param i2c: The I2C bus to use, will try board.I2C()
            if not supplied

        """
        displayio.release_displays()

        if i2c is None:
            i2c = board.I2C()

        # set up on-board seesaw
        self._ss = Seesaw(i2c)

        # set up seesaw pins
        self._ss.pin_mode(SS_TFTRESET_PIN, self._ss.OUTPUT)  # left sceen reset

        # buttons abolve left eye
        self._ss.pin_mode(SS_SWITCH1_PIN, self._ss.INPUT_PULLUP)
        self._ss.pin_mode(SS_SWITCH2_PIN, self._ss.INPUT_PULLUP)
        self._ss.pin_mode(SS_SWITCH3_PIN, self._ss.INPUT_PULLUP)

        # light sensor near left eye
        self._ss.pin_mode(SS_LIGHTSENSOR_PIN, self._ss.INPUT)

        # Manual reset for left screen
        self._ss.digital_write(SS_TFTRESET_PIN, False)
        time.sleep(0.01)
        self._ss.digital_write(SS_TFTRESET_PIN, True)
        time.sleep(0.01)

        # Left backlight pin, on the seesaw
        self._ss.pin_mode(SS_BACKLIGHT_PIN, self._ss.OUTPUT)
        # backlight on full brightness
        self._ss.analog_write(SS_BACKLIGHT_PIN, 255)

        # Left screen spi bus
        left_spi = busio.SPI(board.LEFT_TFT_SCK, MOSI=board.LEFT_TFT_MOSI)
        left_tft_cs = board.LEFT_TFT_CS
        left_tft_dc = board.LEFT_TFT_DC

        left_display_bus = fourwire.FourWire(
            left_spi,
            command=left_tft_dc,
            chip_select=left_tft_cs,  # Reset on Seesaw
        )

        self.left_display = ST7789(left_display_bus, width=240, height=240, rowstart=80)

        # right backlight on board
        self.right_backlight = pwmio.PWMOut(board.RIGHT_TFT_LITE, frequency=5000, duty_cycle=0)
        # full brightness
        self.right_backlight.duty_cycle = 65535

        # right display spi bus
        right_spi = busio.SPI(board.RIGHT_TFT_SCK, MOSI=board.RIGHT_TFT_MOSI)
        right_tft_cs = board.RIGHT_TFT_CS
        right_tft_dc = board.RIGHT_TFT_DC

        right_display_bus = fourwire.FourWire(
            right_spi,
            command=right_tft_dc,
            chip_select=right_tft_cs,
            reset=board.RIGHT_TFT_RST,  # reset on board
        )

        self.right_display = ST7789(right_display_bus, width=240, height=240, rowstart=80)

        # setup accelerometer
        if i2c is not None:
            int1 = digitalio.DigitalInOut(board.ACCELEROMETER_INTERRUPT)
            try:
                self._accelerometer = adafruit_lis3dh.LIS3DH_I2C(i2c, address=0x19, int1=int1)
            except ValueError:
                self._accelerometer = adafruit_lis3dh.LIS3DH_I2C(i2c, int1=int1)

        # touchio on nose
        self.nose = touchio.TouchIn(board.NOSE)

        # can be iffy, depending on environment and person.
        # User code can tweak if needed.
        self.nose.threshold = 180

    @property
    def acceleration(self) -> Union[float, None]:
        """Accelerometer data, +/- 2G sensitivity.

        This example initializes the mask and prints the accelerometer data.

        .. code-block:: python

            import adafruit_monsterm4sk
            mask = adafruit_monsterm4sk.MonsterM4sk(i2c=board.I2C())
            print(mask.acceleration)

        """
        return self._accelerometer.acceleration if self._accelerometer is not None else None

    @property
    def light(self) -> int:
        """Light sensor data.

        This example initializes the mask and prints the light sensor data.

        .. code-block:: python

                import adafruit_monsterm4sk
                mask = adafruit_monsterm4sk.MonsterM4sk(i2c=board.I2C())
                print(mask.light)

        """
        return self._ss.analog_read(SS_LIGHTSENSOR_PIN)

    @property
    def buttons(self) -> Dict[str, bool]:
        """Buttons dictionary.

        This example initializes the mask and prints when the S9 button
        is pressed down.

        .. code-block:: python

                import adafruit_monsterm4sk
                mask = adafruit_monsterm4sk.MonsterM4sk(i2c=board.I2C())

                while True:
                    if mask.buttons["S9"]:
                        print("Button S9 pressed!")

        """

        return {
            "S9": self._ss.digital_read(SS_SWITCH1_PIN) is False,
            "S10": self._ss.digital_read(SS_SWITCH2_PIN) is False,
            "S11": self._ss.digital_read(SS_SWITCH3_PIN) is False,
        }

    @property
    def boop(self) -> bool:
        """Nose touch sense.

        This example initializes the mask and prints when the nose touch pad
        is being touched.

        .. code-block:: python

                import adafruit_monsterm4sk
                mask = adafruit_monsterm4sk.MonsterM4sk(i2c=board.I2C())

                while True:
                    if mask.boop:
                        print("Nose touched!")

        """
        return self.nose.value



# SPDX-FileCopyrightText: 2020 Foamyguy, written for Adafruit Industries
#
# SPDX-License-Identifier: Unlicense

"""
CircuitPython example for Monster M4sk.

Draws star images on each screen. When buttons are pressed
set the stars to a different color. When the nose is booped
make the eyes change through the rainbow.
"""

import time

import adafruit_imageload
import board
import displayio

import adafruit_monsterm4sk

SCREEN_SIZE = 240
IMAGE_SIZE = 64 * 3

i2c_bus = board.I2C()  # uses board.SCL and board.SDA
# i2c_bus = board.STEMMA_I2C()  # For using the built-in STEMMA QT connector on a microcontroller

mask = adafruit_monsterm4sk.MonsterM4sk(i2c=i2c_bus)

left_group = displayio.Group(scale=3)
mask.left_display.root_group = left_group

right_group = displayio.Group(scale=3)
mask.right_display.root_group = right_group

left_group.x = (SCREEN_SIZE - IMAGE_SIZE) // 2
left_group.y = (SCREEN_SIZE - IMAGE_SIZE) // 2

right_group.x = (SCREEN_SIZE - IMAGE_SIZE) // 2
right_group.y = (SCREEN_SIZE - IMAGE_SIZE) // 2

#  load in party parrot bitmap
star_bitmap, star_palette = adafruit_imageload.load(
    "/rainbow_star.bmp", bitmap=displayio.Bitmap, palette=displayio.Palette
)

right_star_grid = displayio.TileGrid(
    star_bitmap,
    pixel_shader=star_palette,
    width=1,
    height=1,
    tile_height=64,
    tile_width=64,
    default_tile=0,
    x=0,
    y=0,
)

left_star_grid = displayio.TileGrid(
    star_bitmap,
    pixel_shader=star_palette,
    width=1,
    height=1,
    tile_height=64,
    tile_width=64,
    default_tile=0,
    x=0,
    y=0,
)

right_group.append(right_star_grid)
left_group.append(left_star_grid)
while True:
    if mask.boop:
        for i in range(6 * 3):
            right_star_grid[0] = i % 6
            left_star_grid[0] = i % 6
            time.sleep(0.02)
        time.sleep(0.5)

    if mask.buttons["S9"]:
        right_star_grid[0] = 2
        left_star_grid[0] = 2

    if mask.buttons["S10"]:
        right_star_grid[0] = 4
        left_star_grid[0] = 4

    if mask.buttons["S11"]:
        right_star_grid[0] = 3
        left_star_grid[0] = 3



# SPDX-FileCopyrightText: 2020 Foamyguy, written for Adafruit Industries
#
# SPDX-License-Identifier: Unlicense

"""
CircuitPython example for Monster M4sk.

Draws a yellow triangle on each screen as Jack-O-Lantern eyes.
The eyes shift back and forth from left to right.
"""

import time

import adafruit_imageload
import board
import displayio

import adafruit_monsterm4sk

SCREEN_SIZE = 240
IMAGE_SIZE = 173

# Create a bitmap for the background 10x10 pixels
bg_color_pixel = displayio.Bitmap(10, 10, 1)

# Create a two color palette
bg_palette = displayio.Palette(2)
bg_palette[0] = 0xFF7700  # orange

bg_color_pixel.fill(0)  # fill orange

# Create a TileGrid for the orange background
bg_left_tile_grid = displayio.TileGrid(bg_color_pixel, pixel_shader=bg_palette)
bg_right_tile_grid = displayio.TileGrid(bg_color_pixel, pixel_shader=bg_palette)

# Create background groups scaled 24x to match screen size
bg_left_group = displayio.Group(scale=24)
bg_right_group = displayio.Group(scale=24)

# add the background tilegrids to the scaled groups
bg_left_group.append(bg_left_tile_grid)
bg_right_group.append(bg_right_tile_grid)

# load the eye image
eye_image, eye_palette = adafruit_imageload.load(
    "/small_triangle_eye.bmp", bitmap=displayio.Bitmap, palette=displayio.Palette
)

# Create a TileGrid to hold the bitmap for each eye
right_pumkin_eye_tilegrid = displayio.TileGrid(eye_image, pixel_shader=eye_palette)
left_pumkin_eye_tilegrid = displayio.TileGrid(eye_image, pixel_shader=eye_palette)

# initialize the monster m4sk hardware
i2c_bus = board.I2C()  # uses board.SCL and board.SDA
# i2c_bus = board.STEMMA_I2C()  # For using the built-in STEMMA QT connector on a microcontroller
mask = adafruit_monsterm4sk.MonsterM4sk(i2c=i2c_bus)

# left eye group setup
left_group = displayio.Group()
mask.left_display.root_group = left_group

# right eye group setup
right_group = displayio.Group()
mask.right_display.root_group = right_group

# Add orange backgrounds to both groups
right_group.append(bg_right_group)
left_group.append(bg_left_group)

# add triangle eyes to both groups
right_group.append(right_pumkin_eye_tilegrid)
left_group.append(left_pumkin_eye_tilegrid)

# centered vertically
right_pumkin_eye_tilegrid.y = (SCREEN_SIZE - IMAGE_SIZE) // 2
left_pumkin_eye_tilegrid.y = (SCREEN_SIZE - IMAGE_SIZE) // 2

while True:
    # move the eyes to the right
    for i in range(0, 240 - 173, 5):
        right_pumkin_eye_tilegrid.x = i
        left_pumkin_eye_tilegrid.x = i
        time.sleep(0.01)

    time.sleep(2)  # wait 2 seconds

    # move the eyes to the left
    for i in range(0, 240 - 173, 5):
        right_pumkin_eye_tilegrid.x -= 5
        left_pumkin_eye_tilegrid.x -= 5
        time.sleep(0.01)

    time.sleep(2)  # wait 2 seconds

    if mask.boop:
        pass

    if mask.buttons["S9"]:
        pass

    if mask.buttons["S10"]:
        pass

    if mask.buttons["S11"]:
        pass
